from ngsolve import *
from netgen.geom2d import *
import netgen.gui

geom = SplineGeometry()
geom.AddRectangle( (0, 0), (2, 1), leftdomain=1, rightdomain=0, bcs = ("wall", "free", "wall", "free") )
geom.AddCircle ( (0.5, 0.5), r=0.25, leftdomain = 2, 
        rightdomain = 1, bc = "cyl" )
ngmesh = geom.GenerateMesh(maxh=0.1)
ngmesh.SetMaterial(1, "f1")
ngmesh.SetMaterial(2, "f2")

k = 1
scale = 2
mesh = Mesh(ngmesh)
mesh.Curve(k)

# deformation
gfd = GridFunction(VectorH1(mesh, order=k))
gfd.Set(CoefficientFunction((scale*x*(2-x)*y*(1-y),0)))

gfu = GridFunction(H1(mesh, order=3, definedon="f2"))
mesh.SetDeformation(gfd)
gfu.Set(sin(10*(x+y)))
mesh.UnsetDeformation

#Draw(gfd, mesh, "deform")
#Draw(gfu, mesh, "gfu")
#from ngsolve.internal import visoptions
#visoptions.deformation = 1
#input("original mesh with 'bad' deformation")

# step 1 (copy mesh and move points): linear mesh
tmpngmesh = mesh.ngmesh.Copy()
for p in tmpngmesh.Points():
    px,py = p[0], p[1]
    diffx = gfd(px,py)[0]
    diffy = gfd(px,py)[1]
    p[0] += diffx
    p[1] += diffy
tmpmesh = Mesh(tmpngmesh)

gftmp = GridFunction(H1(tmpmesh, order=3, definedon="f2"))
gftmp.vec.data = gfu.vec

#visoptions.deformation = 0
#Draw(gftmp, tmpmesh, "gftmp")
#input("bad mesh")

# step 2 (optimize and interpolate) 
mesh2 = Mesh(tmpmesh.ngmesh.Copy())
mesh2.ngmesh.OptimizeMesh2d()
#mesh2.ngmesh.Save("mesh2.vol")
#Draw(mesh2)
#input("good mesh")

gfu2 = GridFunction(H1(mesh2, order=3,definedon="f2"))
gfu2.Set(gftmp)
Draw(gfu2, mesh2, "gfu2")
input("good mesh")
